@@ -6,7 +6,7 @@ Sys::Statistics::Linux - Front-end module to collect system statistics
 
    use Sys::Statistics::Linux;
 
-   my $lxs = new Sys::Statistics::Linux;
+   my $lxs = Sys::Statistics::Linux->new();
 
    $lxs->set(
       SysInfo   => 1,
@@ -29,30 +29,36 @@ Sys::Statistics::Linux - Front-end module to collect system statistics
 
 =head1 DESCRIPTION
 
-Sys::Statistics::Linux is the front-end module to Sys-Statistics-Linux and collects
-different linux system informations like processor workload, memory usage, network and
-disk statistics and a lot more. Refer the documentation of the distribution
-modules to get more informations about all possible statistics.
+Sys::Statistics::Linux is a front-end module and gather different linux system informations
+like processor workload, memory usage, network and disk statistics and a lot more. Refer the
+documentation of the distribution modules to get more informations about all possible statistics.
+
+=head1 MOTIVATION
+
+My motivation is very simple. Every linux administrator knows the well-known tool sar of sysstat.
+It helps me a lot of time to search for system bottlenecks and to solve problems but it's hard to
+parse the output to store different statistics into a database. So I though to develope
+Sys::Statistics::Linux. It's not a replacement but it should make it simpler to you to write your
+own system monitor.
+
+If Sys::Statistics::Linux doesn't provide statistics that are strongly needed then let me know it.
 
 =head1 TECHNICAL NOTE
 
-This distribution collects statistics by the virtual F</proc> filesystem (procfs) and is developed
-on default vanilla kernels. It is tested on x86 hardware with the distributions SuSE (SuSE on s390
-and s390x architecture as well), Red Hat, Debian, Asianux, Slackware and Mandrake on kernel versions
-2.4 and 2.6 and should run on all linux kernels with a default vanilla kernel as well.
-It is possible that this module doesn't run on all distributions if the procfs is too much modified.
-
-For example the linux kernel 2.4 can compiled with the option "CONFIG_BLK_STATS". It is possible to
-activate or deactivate the block statistics for devices with this option. These statistics doesn't
-exist in /proc/partitions if this option isn't activated. Since linux kernel 2.5 these statistics are
-in /proc/diskstats.
+This distribution collects statistics by the virtual F</proc> filesystem (procfs) and is
+developed on default vanilla kernels. It is tested on x86 hardware with the distributions SuSE
+(SuSE on s390 and s390x architecture as well), openSUSE, RHLE, Fedora, Debian, Ubuntu, Asianux,
+Slackware and Mandriva on kernel versions 2.4 and/or 2.6 and should run on all linux kernels with
+a default vanilla kernel as well. It's possible that it doesn't run on all linux distributions
+if some procfs features are deactived or too much modified. As example the linux kernel 2.4 can
+compiled with the option C<CONFIG_BLK_STATS> what turn on or off block statistics for devices.
 
 =head1 DELTAS
 
 The statistics C<CpuStats>, C<ProcStats>, C<PgSwStats>, C<NetStats>, C<DiskStats> and C<Processes>
 are deltas, for this reason it's necessary to initialize the statistics first before the data
 can be prepared by C<get()>. These statistics can be initialized with the methods C<new()>, C<set()>
-and C<init()>. Each option that is set to TRUE (1) will be initialized by the call of C<new()>
+and C<init()>. Any option that is set to TRUE (1) will be initialized by the call of C<new()>
 or C<set()>. The call of C<init()> reinitialize all statistics that are set to 1. By the call
 of C<get()> the initial statistics will be updated automatically. Please refer the METHOD section
 to get more information about the calls of C<new()>, C<set()> and C<get()>.
@@ -74,8 +80,7 @@ C<init()>.
 
 The options must be set with on of the following values:
 
-   -1 - set pause on statistics but wouldn't delete them
-    0 - delete statistics and destroy the object
+    0 - deactivate statistics
     1 - create a new object and init statistics if necessary
     2 - create a new object if not exists but wouldn't init statistics
 
@@ -88,9 +93,6 @@ In addition it's possible to handoff a process list for option C<Processes>.
        }
    );
 
-It's only possible to set C<init> to 1 or 2. 1 would init statistics, 2 not but both values would
-create a new C<Processes> object.
-
 To get more informations about the statistics refer the different modules of the distribution.
 
    SysInfo     -  Collect system informations             with Sys::Statistics::Linux::SysInfo.
@@ -115,7 +117,7 @@ This options would be handoff to the method C<set()>.
 
 Without options
 
-         my $lxs = new Sys::Statistics::Linux;
+         my $lxs = Sys::Statistics::Linux->new();
 
 Or with options
 
@@ -138,12 +140,12 @@ Take a look to C<set()> for more informations.
 
 =head2 set()
 
-Call C<set()> to activate or deactivate options. The following example would call C<new()> and C<init()>
-of C<Sys::Statistics::Linux::CpuStats> and delete the object of C<Sys::Statistics::Linux::SysInfo>:
+Call C<set()> to activate or deactivate options. The following example would call C<new()> and
+C<init()> of C<Sys::Statistics::Linux::CpuStats> and delete the object of
+C<Sys::Statistics::Linux::SysInfo>:
 
          $lxs->set(
-            CpuStats  => -1, # activated, but paused, wouldn't delete the object
-            Processes =>  0, # deactivate - would delete the statistics and destroy the object
+            Processes =>  0, # deactivate this statistic
             PgSwStats =>  1, # activate the statistic and calls new() and init() if necessary
             NetStats  =>  2, # activate the statistic and call new() if necessary but not init()
          );
@@ -165,7 +167,8 @@ Call C<get()> to get the collected statistics. C<get()> returns the statistics a
 
 =head2 init()
 
-The call of C<init()> re-init all statistics that are necessary for deltas and if the option is higher than 0.
+The call of C<init()> re-init all statistics that are necessary for deltas and if the option is
+higher than 0.
 
          $lxs->init;
 
@@ -203,7 +206,8 @@ This would return the following matches:
     * all cpu where "iowait" is grather than 10
     * only disk '/dev/sda1' if "usageper" is grather than 80
 
-If the statistics are not gathered by the current process then you can handoff statistics as an argument.
+If the statistics are not gathered by the current process then you can handoff statistics as an
+argument.
 
         my %stats = (
            CpuStats => {
@@ -226,8 +230,8 @@ If the statistics are not gathered by the current process then you can handoff s
 
         my $hits = $lxs->search(\%filter, \%stats);
 
-The method C<psfind()> scans for processes only and returns a array reference with all process IDs that
-matched the filter. Example:
+The method C<psfind()> scans for processes only and returns a array reference with all process
+IDs that matched the filter. Example:
 
         my $pids = $lxs->psfind({ cmd => qr/init/, owner => 'eq:apache' });
 
@@ -272,8 +276,8 @@ or the manpage C<strftime(3)>.
 =head2 gettime()
 
 C<gettime()> returns a POSIX formatted time stamp, @foo in list and $bar in scalar context.
-If the time format isn't set then the default format "%Y-%m-%d %H:%M:%S" will be set automatically.
-You can also set a time format with C<gettime()>.
+If the time format isn't set then the default format "%Y-%m-%d %H:%M:%S" will be set
+automatically. You can also set a time format with C<gettime()>.
 
          my $date_time = $lxs->gettime;
 
@@ -306,82 +310,13 @@ A very simple perl script could looks like this:
          print "  ioWait    $cpu->{iowait}\n";
          print "  total     $cpu->{total}\n";
 
-Example to collect network statistics with a nice output:
-
-         use warnings;
-         use strict;
-         use Sys::Statistics::Linux;
-
-         $| = 1;
-
-         my $header  = 20;
-         my $average = 1;
-         my $columns = 8;
-         my $options = { NetStats => 1 };
-
-         my @order = qw(
-            rxbyt rxpcks rxerrs rxdrop rxfifo rxframe rxcompr rxmulti
-            txbyt txpcks txerrs txdrop txfifo txcolls txcarr txcompr
-         );
-
-         my $lxs = Sys::Statistics::Linux->new( $options );
-
-         my $h = $header;
-
-         while (1) {
-            sleep($average);
-            my $stats = $lxs->get;
-            if ($h == $header) {
-               printf "%${columns}s", $_ for ('iface', @order);
-               print "\n";
-            }
-            foreach my $device (keys %{$stats->{NetStats}}) {
-               my $dstat = $stats->{NetStats}->{$device};
-               printf "%${columns}s", $device;
-               printf "%${columns}s", $dstat->{$_} for @order;
-               print "\n";
-            }
-            $h = $header if --$h == 0;
-         }
-
-Activate and deactivate statistics:
-
-         use warnings;
-         use strict;
-         use Sys::Statistics::Linux;
-         use Data::Dumper;
-
-         my $lxs = new Sys::Statistics::Linux;
-
-         # set the options
-         $lxs->set(
-            SysInfo  => 1,
-            CpuStats => 1,
-            MemStats => 1
-         );
-
-         # sleep to get useful statistics for CpuStats
-         sleep(1);
-
-         # $stats contains SysInfo, CpuStats and MemStats
-         my $stats = $lxs->get;
-         print Dumper($stats);
-
-         # we deactivate CpuStats
-         $lxs->set(SysStats => 0);
-
-         # $stats contains CpuStats and MemStats
-         sleep(1);
-         $stats = $lxs->get;
-         print Dumper($stats);
-
 Set and get a time stamp:
 
          use warnings;
          use strict;
          use Sys::Statistics::Linux;
 
-         my $lxs = new Sys::Statistics::Linux;
+         my $lxs = Sys::Statistics::Linux->new();
          $lxs->settime('%Y/%m/%d %H:%M:%S');
          print "$lxs->gettime\n";
 
@@ -415,10 +350,10 @@ How to get processes with the highest cpu workload:
             map  { [ $_, $procs->{$_}->{ttime} ] } keys %{$procs}
          )[0..4];
 
-Take a look into the the F<examples> directory of the distribution for some examples with a nice output. :-)
-
 =head1 DEPENDENCIED
 
+    UNIVERSAL
+    UNIVERSAL::require
     Test::More
     Carp
 
@@ -428,7 +363,6 @@ No exports.
 
 =head1 TODOS
 
-   * pstree
    * Are there any wishs from your side? Send me a mail!
 
 =head1 REPORTING BUGS
@@ -448,18 +382,19 @@ This program is free software; you can redistribute it and/or modify it under th
 =cut
 
 package Sys::Statistics::Linux;
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use strict;
 use warnings;
 use Carp qw(croak);
 use POSIX qw(strftime);
-use constant RXOPTION => qr/^[0-2\-1]\z/;
+use UNIVERSAL;
+use UNIVERSAL::require;
 
 sub new {
    my $class = shift;
    my $self = bless {
-      opts  => {
+      opts => {
          SysInfo   =>  0,
          CpuStats  =>  0,
          ProcStats =>  0,
@@ -484,56 +419,52 @@ sub new {
 sub set {
    my $self  = shift;
    my $class = ref $self;
-   my $sets  = $class->_struct(@_);
+   my $args  = ref($_[0]) eq 'HASH' ? shift : {@_};
    my $opts  = $self->{opts};
    my $obj   = $self->{obj};
    my $stats = $self->{stats};
    my $pids  = ();
 
-   if (ref($sets->{Processes}) eq 'HASH') {
-      $pids = $sets->{Processes}->{pids};
-      $sets->{Processes} = $sets->{Processes}->{init};
+   if (ref($args->{Processes}) eq 'HASH') {
+      $pids = $args->{Processes}->{pids};
+      $args->{Processes} = $args->{Processes}->{init};
    }
 
-   foreach my $opt (keys %{$sets}) {
+   foreach my $opt (keys %{$args}) {
 
       # validate the options
       croak "$class: invalid option '$opt'"
          unless exists $opts->{$opt};
       croak "$class: invalid value for '$opt'"
-         unless $sets->{$opt} =~ RXOPTION;
+         unless $args->{$opt} =~ qr/^[012]\z/;
 
-      $opts->{$opt} = $sets->{$opt};
+      $opts->{$opt} = $args->{$opt};
 
-      if ($opts->{$opt} > 0) {
+      if ($opts->{$opt}) {
          my $package = $class."::".$opt;
 
          # require mod if not loaded
-         unless (defined &{$package.'::new'}) {
-            my $require = $package;
-            $require =~ s/::/\//g;
-            require "$require.pm"
-               or croak "$class: unable to load $require.pm";
+         unless ($obj->{$opt}) {
+            $package->require
+               or croak "$class: unable to load $package";
          }
 
          # create a new object if the object doesn't exist
          # or create a new process list object if $pids is set
          if ($opt eq 'Processes' && $pids) {
             $obj->{$opt} = $package->new($pids);
-         } else {
-            $obj->{$opt} = $package->new
-               unless $obj->{$opt};
+         } elsif (!$obj->{$opt}) {
+            $obj->{$opt} = $package->new;
          }
 
-         # get initial statistics if init() is defined and the
-         # option is set to 1
+         # get initial statistics if the function init() exists
+         # and the option is set to 1
          $obj->{$opt}->init() 
             if $opts->{$opt} == 1
-            && defined &{$package.'::init'};
+            && UNIVERSAL::can($package, 'init');
 
-      } elsif ($obj->{$opt} && $opts->{$opt} == 0) {
-         delete $obj->{$opt}   if $obj->{$opt};
-         delete $stats->{$opt} if $stats->{$opt};
+      } elsif ($stats->{$obj}) {
+         delete $stats->{$obj};
       }
    }
 }
@@ -545,9 +476,8 @@ sub init {
    my $opts  = $self->{opts};
 
    foreach my $opt (keys %{$opts}) {
-      my $package = $class."::".$opt;
       $obj->{$opt}->init()
-         if defined &{$package.'::init'}
+         if defined &{"$class::$opt::init"}
          && $opts->{$opt} > 0;
    }
 }
@@ -570,11 +500,10 @@ sub get {
    my $opts   = $self->{opts};
    my $stats  = $self->{stats};
    my $obj    = $self->{obj};
-   my $format = $self->{timeformat};
 
    foreach my $opt (keys %{$opts}) {
       $stats->{$opt} = $obj->{$opt}->get()
-         if $opts->{$opt} > 0;
+         if $opts->{$opt};
    }
 
    return $stats;
@@ -616,7 +545,7 @@ sub search {
 
       foreach my $x (keys %{$fref}) {
 
-         # if $fref->{$x} is a hash ref than the next key have to
+         # if $fref->{$x} is a hash ref then the next key have to
          # match the statistic key. this is used for statistics
          # like NetStats or Processes that uses a hash key for the
          # device name or process id. NetStats example:
@@ -625,16 +554,18 @@ sub search {
          #
          #    $fref->{eth0}->{ttbyt}
          #
-         # is defined as a filter than the key "eth0" have to match
+         # is defined as a filter then the key "eth0" have to match
          #
          #    $sref->{eth0}
          #
-         # than we look if "ttbyt" matched the searched string
+         # then we look if "ttbyt" matched the searched statistic name
+         # and compare the values. if the comparing returns TRUE we the
+         # hash is copied until the matched key-value pair.
 
          if (ref($fref->{$x}) eq 'HASH') {
 
             # if the key $sref->{eth0} doesn't exists
-            # than we continue with the next defined filter
+            # then we continue with the next defined filter
             next unless exists $sref->{$x};
 
             while ( my ($name, $value) = each %{$fref->{$x}} ) {
@@ -642,14 +573,13 @@ sub search {
                   if exists $sref->{$x}->{$name}
                   && $class->_compare($sref->{$x}->{$name}, $value);
             }
-
          } else {
             foreach my $key (keys %{$sref}) {
                if (ref($sref->{$key}) eq 'HASH') {
                   $hits{$opt}{$key}{$x} = $sref->{$key}->{$x}
                      if exists $sref->{$key}->{$x}
                      && $class->_compare($sref->{$key}->{$x}, $fref->{$x});
-               } else {
+               } else { # must be a scalar now
                   $hits{$opt}{$x} = $sref->{$x}
                      if exists $sref->{$x}
                      && $class->_compare($sref->{$x}, $fref->{$x});
@@ -707,28 +637,15 @@ sub _compare {
       return $x ne $y;
    } elsif ($y =~ s/^gt://) {
       return $x > $y if $y =~ /^\d+\z/;
-      croak "$class: not a number for operator 'gt:'";
+      croak "$class: not a number for operator 'gt'";
    } elsif ($y =~ s/^lt://) {
       return $x < $y if $y =~ /^\d+\z/;
-      croak "$class: not a number for operator 'lt:'";
+      croak "$class: not a number for operator 'lt'";
    } else {
-      croak "$class: bad search() or psfind() operator '$y'";
+      croak "$class: bad search() / psfind() operator '$y'";
    }
 
    return undef;
 }
 
-sub _struct {
-   my $class = shift;
-
-   return
-      ref($_[0])
-         ? ref($_[0]) eq 'HASH'
-            ? $_[0]
-            : croak "not a hash ref"
-         : @_ % 2
-            ? croak "odd number of elements in hash"
-            : {@_};
-}
-
 1;
